@@ -1,5 +1,5 @@
-# pylint: disable-msg=W0622
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# pylint: disable-msg=W0622,C0103
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -16,17 +16,17 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """pylint packaging information"""
 
-__revision__ = '$Id: __pkginfo__.py,v 1.41 2005/11/07 14:41:13 syt Exp $'
+__revision__ = '$Id: __pkginfo__.py,v 1.50 2006-04-19 09:17:40 syt Exp $'
 
 
 modname = 'pylint'
 
-numversion = (0, 8, 1)
+numversion = (0, 11, 0)
 version = '.'.join([str(num) for num in numversion])
 
 license = 'GPL'
-copyright = '''Copyright (c) 2003-2005 Sylvain Thenault (thenault@gmail.com).
-Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+copyright = '''Copyright (c) 2003-2006 Sylvain Thenault (thenault@gmail.com).
+Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr'''
 
 short_desc = "python code static checker"
@@ -55,9 +55,20 @@ from os.path import join
 scripts = [join('bin', filename)
            for filename in ('pylint', 'pylint-gui', "symilar")]
 
-include_dirs = [join('test', 'input'), join('test', 'messages')]
+include_dirs = [join('test', 'input'), join('test', 'messages'),
+                join('test', 'regrtest_data')]
 
 pyversions = ["2.2", "2.3", "2.4"]
 
-debian_maintainer = 'Alexandre Fayolle'
-debian_maintainer_email = 'afayolle@debian.org'
+debian_uploader = 'Alexandre Fayolle <afayolle@debian.org>'
+
+classifiers =  ['Development Status :: 4 - Beta',
+                'Environment :: Console',
+                'Intended Audience :: Developers',
+                'License :: OSI Approved :: GNU General Public License (GPL)',
+                'Operating System :: OS Independent',
+                'Programming Language :: Python',
+                'Topic :: Software Development :: Debuggers',
+                'Topic :: Software Development :: Quality Assurance',
+                'Topic :: Software Development :: Testing',
+                ]
