@@ -14,9 +14,21 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """utilities methods and classes for checkers
+
+Base id of standard checkers (used in msg and report ids):
+01: base
+02: classes
+03: format
+04: import
+05: misc
+06: variables
+07: exceptions
+08: similar
+09: design_analysis
+10: newstyle
 """
 
-__revision__ = "$Id: __init__.py,v 1.20 2005/11/02 09:21:45 syt Exp $"
+__revision__ = "$Id: __init__.py,v 1.21 2005/11/21 23:08:11 syt Exp $"
 
 import tokenize
 from os import listdir
