@@ -10,15 +10,13 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-""" Copyright (c) 2002-2005 LOGILAB S.A. (Paris, FRANCE).
+""" Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 
  basic checker for Python code
-
- FIXME : should check constant names !
 """
 
-__revision__ = "$Id: base.py,v 1.58 2005/11/03 09:21:28 syt Exp $"
+__revision__ = "$Id: base.py,v 1.65 2006-01-26 00:26:23 syt Exp $"
 
 from logilab import astng
 from logilab.common.ureports import Table
@@ -32,17 +30,23 @@ import re
 
 # regex for class/function/variable/constant nane
 CLASS_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
-FUNC_NAME_RGX = re.compile('[a-z_][a-z0-9_]*$')
-METH_NAME_RGX = re.compile('[a-z_][a-z0-9_]*$')
-VAR_NAME_RGX = re.compile('[a-z_][a-z0-9_]*$')
 MOD_NAME_RGX = re.compile('(([a-z_][a-z0-9_]*)|([A-Z][a-zA-Z0-9]+))$')
-#CST_NAME_RGX = re.compile('[A-Z_][A-Z1-9_]*')
-
+CONST_NAME_RGX = re.compile('(([A-Z_][A-Z1-9_]*)|(__.*__))$')
+COMP_VAR_RGX = re.compile('[A-Za-z_][A-Za-z0-9_]*$')
+DEFAULT_NAME_RGX = re.compile('[a-z_][a-z0-9_]{2,30}$')
 # do not require a doc string on system methods
 NO_REQUIRED_DOC_RGX = re.compile('__.*__')
 
 del re
 
+def in_loop(node):
+    parent = node.parent
+    while parent is not None:
+        if isinstance(parent, (astng.For, astng.ListComp, astng.GenExpr)):
+            return True
+        parent = parent.parent
+    return False
+
 def in_nested_list(nested_list, obj):
     """return true if the object is an element of <nested_list> or of a nested
     list
@@ -106,24 +110,14 @@ MSGS = {
     'W0102': ('Dangerous default value %s as argument',
               'Used when a mutable value as list or dictionary is detected in \
               a default value for an argument.'),
-    'W0103': ('Missing required attribute "%s"',
-              'Used when an attribute required for modules is missing.'),
-    
-    'W0121': ('Use of the global statement',
-              'Used when you use the "global" statement, to discourage its \
-              usage. That doesn\'t mean you can not use it !'),
+    'W0104': ('Statement seems to have no effect',
+              'Used when a statement doesn\'t have (or at least seems to) \
+              any effect.'),
+
     'W0122': ('Use of the exec statement',
               'Used when you use the "exec" statement, to discourage its \
               usage. That doesn\'t mean you can not use it !'),
     
-    'W0131': ('Missing docstring',
-              'Used when a module, function, class or method has no docstring.\
-              Some special methods like __init__ doesn\'t necessary require a \
-              docstring.'),
-    'W0132': ('Empty docstring',
-              'Used when a module, function, class or method has an empty \
-              docstring (it would be to easy ;).'),
-
     'W0141': ('Used builtin function %r',
               'Used when a black listed builtin function is used (see the \
               bad-function option). Usual black listed functions are the ones \
@@ -134,8 +128,6 @@ MSGS = {
               `**kwargs` to dispatch arguments. This doesn\'t improve readility\
                and should be used with care.'),
 
-    'C0101': ('Too short name "%s"',
-              'Used when a variable has a too short name.'),
     'C0102': ('Black listed name "%s"',
               'Used when the name is listed in the black list (unauthorized \
               names).'),
@@ -143,6 +135,17 @@ MSGS = {
               'Used when the name doesn\'t match the regular expression \
               associated to its type (constant, variable, class...).'),
     
+    'C0111': ('Missing docstring', # W0131
+              'Used when a module, function, class or method has no docstring.\
+              Some special methods like __init__ doesn\'t necessary require a \
+              docstring.'),
+    'C0112': ('Empty docstring', # W0132
+              'Used when a module, function, class or method has an empty \
+              docstring (it would be to easy ;).'),
+
+    'C0121': ('Missing required attribute "%s"', # W0103
+              'Used when an attribute required for modules is missing.'),
+    
     }
 
 class BasicChecker(BaseChecker):
@@ -175,17 +178,23 @@ functions, methods
                           'functions or classes name which do not require a '
                           'docstring'}
                 ),
-               ('min-name-length',
-                {'default' : 3, 'type' : 'int', 'metavar' : '<int>',
-                 'help': 'Minimal length for module / class / function / '
-                         'method / argument / variable names'}
-                ),
+##                ('min-name-length',
+##                 {'default' : 3, 'type' : 'int', 'metavar' : '<int>',
+##                  'help': 'Minimal length for module / class / function / '
+##                          'method / argument / variable names'}
+##                 ),
                ('module-rgx',
                 {'default' : MOD_NAME_RGX,
                  'type' :'regexp', 'metavar' : '<regexp>',
                  'help' : 'Regular expression which should only match correct '
                           'module names'}
                 ),
+               ('const-rgx',
+                {'default' : CONST_NAME_RGX,
+                 'type' :'regexp', 'metavar' : '<regexp>',
+                 'help' : 'Regular expression which should only match correct '
+                          'module level names'}
+                ),
                ('class-rgx',
                 {'default' : CLASS_NAME_RGX,
                  'type' :'regexp', 'metavar' : '<regexp>',
@@ -193,28 +202,41 @@ functions, methods
                           'class names'}
                 ),
                ('function-rgx',
-                {'default' : FUNC_NAME_RGX,
+                {'default' : DEFAULT_NAME_RGX,
                  'type' :'regexp', 'metavar' : '<regexp>',
                  'help' : 'Regular expression which should only match correct '
                           'function names'}
                 ),
                ('method-rgx',
-                {'default' : METH_NAME_RGX,
+                {'default' : DEFAULT_NAME_RGX,
                  'type' :'regexp', 'metavar' : '<regexp>',
                  'help' : 'Regular expression which should only match correct '
                           'method names'}
                 ),
+               ('attr-rgx',
+                {'default' : DEFAULT_NAME_RGX,
+                 'type' :'regexp', 'metavar' : '<regexp>',
+                 'help' : 'Regular expression which should only match correct '
+                          'instance attribute names'}
+                ),
                ('argument-rgx',
-                {'default' : VAR_NAME_RGX,
+                {'default' : DEFAULT_NAME_RGX,
                  'type' :'regexp', 'metavar' : '<regexp>',
                  'help' : 'Regular expression which should only match correct '
                           'argument names'}),
                ('variable-rgx',
-                {'default' : VAR_NAME_RGX,
+                {'default' : DEFAULT_NAME_RGX,
                  'type' :'regexp', 'metavar' : '<regexp>',
                  'help' : 'Regular expression which should only match correct '
                           'variable names'}
                 ),
+               ('inlinevar-rgx',
+                {'default' : COMP_VAR_RGX,
+                 'type' :'regexp', 'metavar' : '<regexp>',
+                 'help' : 'Regular expression which should only match correct '
+                          'list comprehension / generator expression variable \
+                          names'}
+                ),
                ('good-names',
                 {'default' : ('i', 'j', 'k', 'ex', 'Run', '_'),
                  'type' :'csv', 'metavar' : '<names>',
@@ -246,11 +268,14 @@ functions, methods
         """initialize visit variables and statistics
         """
         self._returns = []
-        self.stats = self.linter.add_stats(module=0, constant=0, function=0,
-                                           method=0, class_=0, badname_module=0,
+        self.stats = self.linter.add_stats(module=0, function=0,
+                                           method=0, class_=0,
+                                           badname_module=0,
                                            badname_class=0, badname_function=0,
-                                           badname_method=0, badname_constant=0,
+                                           badname_method=0, badname_attr=0,
+                                           badname_const=0,
                                            badname_variable=0,
+                                           badname_inlinevar=0,
                                            badname_argument=0,
                                            undocumented_module=0,
                                            undocumented_function=0,
@@ -274,7 +299,14 @@ functions, methods
         if self.config.no_docstring_rgx.match(node.name) is None:
             self._check_docstring('class', node)
         self._check_redefinition('class', node)
-            
+        for attr, anodes in node.instance_attrs.items():
+            self._check_name('attr', attr, anodes[0])
+
+    def visit_discard(self, node):
+        """check for statement without effect"""
+        if not isinstance(node.expr, astng.CallFunc):
+            self.add_message('W0104', node=node)
+
     def visit_function(self, node):
         """check function name, docstring, arguments, redefinition,
         variable names, max locals
@@ -294,16 +326,6 @@ functions, methods
         args = node.argnames
         if args is not None:
             self._recursive_check_names(args, node)
-        # check local variable, avoiding argument, imported names, global names
-        # and current class name if the function is actually a method
-        for var, stmt in node.items():
-            if (not in_nested_list(args or (), var)
-                and not isinstance(stmt, astng.Import) 
-                and not isinstance(stmt, astng.From) 
-                and not isinstance(stmt, astng.Global) 
-                and not isinstance(stmt, astng.Class) 
-                and not (is_method and var == node.parent.frame().name)):
-                self._check_name('variable', var, stmt)
         # check for redefinition
         self._check_redefinition(is_method and 'method' or 'function', node)
 
@@ -315,6 +337,20 @@ functions, methods
         if node.is_method() and node.name == '__init__' and nb_returns:
             self.add_message('E0101', node=node)
 
+    def visit_assname(self, node):
+        """check module level assigned names"""
+        frame = node.frame()
+        ass_type = node.ass_type()
+        if isinstance(ass_type, (astng.ListCompFor, astng.GenExprFor)):
+            self._check_name('inlinevar', node.name, node)
+        elif isinstance(frame, astng.Module):
+            if isinstance(ass_type, astng.Assign) and not in_loop(ass_type):
+                self._check_name('const', node.name, node)
+        elif isinstance(frame, astng.Function):
+            # global introduced variable aren't in the function locals
+            if node.name in frame:
+                self._check_name('variable', node.name, node)
+    
     def visit_return(self, node):
         """check is the node has a right sibling (if so, that's some unreachable
         code)
@@ -348,10 +384,6 @@ functions, methods
         """
         self._check_unreachable(node)
 
-    def visit_global(self, node):
-        """just print a warning on global statements"""
-        self.add_message('W0121', node=node)
-        
     def visit_exec(self, node):
         """just pring a warning on exec statements"""
         self.add_message('W0122', node=node)
@@ -400,10 +432,10 @@ functions, methods
         docstring = node.doc
         if docstring is None:
             self.stats['undocumented_'+node_type] += 1
-            self.add_message('W0131', node=node)
+            self.add_message('C0111', node=node)
         elif not docstring.strip():
             self.stats['undocumented_'+node_type] += 1
-            self.add_message('W0132', node=node)
+            self.add_message('C0112', node=node)
             
     def _recursive_check_names(self, args, node):
         """check names in a possibly recursive list <arg>"""
@@ -425,21 +457,16 @@ functions, methods
         if regexp.match(name) is None:
             self.add_message('C0103', node=node, args=(name, regexp.pattern))
             self.stats['badname_' + node_type] += 1
-        elif len(name) < self.config.min_name_length:
-            self.add_message('C0101', node=node, args=name)
-            self.stats['badname_' + node_type] += 1
+            
 
     def _check_defaults(self, node):
         """check for dangerous default values as arguments"""
         for default in node.defaults:
-            if default.__class__ is astng.Name:
-                try:
-                    value = node.resolve(default.name)
-                except astng.ResolveError:
-                    continue
-            else:
-                value = default
-            if value.__class__ in (astng.Dict, astng.List):
+            try:
+                value = default.infer().next()
+            except astng.InferenceError:
+                continue
+            if isinstance(value, (astng.Dict, astng.List)):
                 if value is default:
                     msg = default.as_string()
                 else:
@@ -450,7 +477,7 @@ functions, methods
         """check for required attributes"""
         for attr in attributes:
             if not node.has_key(attr):
-                self.add_message('W0103', node=node, args=attr)
+                self.add_message('C0121', node=node, args=attr)
 
     
 def register(linter):
