@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -17,32 +17,34 @@
 """
 from __future__ import generators
 
-__revision__ = "$Id: classes.py,v 1.63 2005/11/02 09:21:46 syt Exp $"
+__revision__ = "$Id: classes.py,v 1.77 2006-03-05 14:39:37 syt Exp $"
 
 from logilab import astng
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
-
+from pylint.checkers.utils import overrides_a_method
 
 MSGS = {
-    'F0201': ('Unable to check method %r of interface %s',
-              'Used when PyLint has been unable to fetch a method declared in \
-              an interface (either in the class or in the interface) and so to\
-              check its implementation.'),
+##     'F0201': ('Unable to check method %r of interface %s',
+##               'Used when PyLint has been unable to fetch a
+    ##method declared in \
+##               an interface (either in the class or in the
+    ##interface) and so to\
+##               check its implementation.'),
     'F0202': ('Unable to check methods signature (%s / %s)',
               'Used when PyLint has been unable to check methods signature \
               compatibility for an unexpected raison. Please report this kind \
               if you don\'t make sense of it.'),
-    'F0203': ('Unable to resolve %s',
-              'Used when PyLint has been unable to resolve a name.'),
-    'F0204': ('Name %s has not been resolved to a class as expected',
-              'Used when PyLint try to resolve an ancestor class name but \
-              gets something else than a Class node.'),
+##     'F0203': ('Unable to resolve %s',
+##               'Used when PyLint has been unable to resolve a name.'),
+##     'F0204': ('Name %s has not been resolved to a class as expected',
+##               'Used when PyLint try to resolve an ancestor class name but \
+##               gets something else than a Class node.'),
 
-    'E0201': ('Access to undefined member %r',
-              'Used when an instance member not defined in the instance, its\
-              class or its ancestors is accessed.'),
+##     'E0201': ('Access to undefined member %r',
+##               'Used when an instance member not defined in the instance, its\
+##               class or its ancestors is accessed.'),
     'E0202': ('An attribute inherited from %s hide this method',
               'Used when a class defines a method which is hiden by an \
               instance attribute from an ancestor class.'),
@@ -56,25 +58,25 @@ MSGS = {
     'E0211': ('Method has no argument',
               'Used when a method which should have the bound instance as \
               first argument has no argument defined.'),
-    'E0212': ('Class method should have "cls" as first argument',
-              'Used when a class method has an attribute different than "cls"\
-              as first argument, to easily differentiate them from regular \
-              instance methods.'),
     'E0213': ('Method should have "self" as first argument',
               'Used when a method has an attribute different the "self" as\
               first argument.'),
-    'E0214': ('Metaclass method should have "mcs" as first argument',
+    'C0202': ('Class method should have "cls" as first argument', # E0212
+              'Used when a class method has an attribute different than "cls"\
+              as first argument, to easily differentiate them from regular \
+              instance methods.'),
+    'C0203': ('Metaclass method should have "mcs" as first argument', # E0214
               'Used when a metaclass method has an attribute different the \
               "mcs" as first argument.'),
     'W0211': ('Static method with %r as first argument',
               'Used when a static method has "self" or "cls" as first argument.'
               ),
-    'W0212': ('Method could be a function',
+    'R0201': ('Method could be a function',
               'Used when a method doesn\'t use its bound instance, and so could\
               be written as a function.'
               ),
     
-    'E0221': ('Interface %s is not a class (%s)',
+    'E0221': ('Interface resolved to %s is not a class',
               'Used when a class claims to implement an interface which is not \
               a class.'),
     'E0222': ('Missing method %r from %s interface' ,
@@ -90,6 +92,10 @@ MSGS = {
               'Used when an abstract method (ie raise NotImplementedError) is \
               not overriden in concrete class.'
               ),
+    'F0220': ('failed to resolve interfaces implemented by %s (%s)', # W0224
+              'Used when a PyLint as failed to find interfaces implemented by \
+               a class'),
+    
     
     'W0231': ('__init__ method from base class %r is not called',
               'Used when an ancestor class method has an __init__ method \
@@ -140,13 +146,6 @@ separated by a comma. This is used for instance to not check methods defines \
 in Zope\'s Interface base class.'}
                 ),
                
-               ('ignore-mixin-members',
-                {'default' : True, 'type' : 'yn', 'metavar': '<y_or_n>',
-                 'help' : 'Tells wether missing members accessed in mixin \
-class should be ignored. A mixin class is detected if its name ends with \
-"mixin" (case insensitive).'}
-                ),
-               
                ('defining-attr-methods',
                 {'default' : ('__init__', '__new__', 'setUp'),
                  'type' : 'csv',
@@ -155,20 +154,6 @@ class should be ignored. A mixin class is detected if its name ends with \
 instance attributes.'}
                 ),
 
-               ('zope',
-                {'default' : False, 'type' : 'yn', 'metavar': '<y_or_n>',
-                 'help' : 'When zope mode is activated, consider the \
-acquired-members option to ignore access to some undefined attributes.'}
-                ),
-               ('acquired-members',
-                {'default' : (
-        'REQUEST', 'acl_users', 'aq_parent'),
-                 'type' : 'csv',
-                 'metavar' : '<members names>',
-                 'help' : 'List of members which are usually get through \
-zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
-(need zope=yes to be considered.'}
-                ),
                )
 
     def __init__(self, linter=None):
@@ -197,12 +182,13 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
         """
         # checks attributes are defined in an allowed method such as __init__
         defining_methods = self.config.defining_attr_methods
-        for attr, node in cnode.instance_attrs.items():
+        for attr, nodes in cnode.instance_attrs.items():
+            node = nodes[0] # XXX
             frame = node.frame()
             if frame.name not in defining_methods:
                 # check attribute is defined in a parent's __init__
                 for parent in cnode.instance_attr_ancestors(attr):
-                    frame = parent.instance_attrs[attr].frame()
+                    frame = parent.instance_attrs[attr][0].frame() # XXX
                     if frame.name in defining_methods:
                         # we're done :)
                         break
@@ -214,9 +200,7 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
                         self.add_message('W0201', args=attr, node=node)
         # check access to existant members on non metaclass classes
         accessed = self._accessed.pop()
-        if cnode.type != 'metaclass' and (
-            not self.config.ignore_mixin_members
-            or cnode.name[-5:].lower() != 'mixin'):
+        if cnode.type != 'metaclass':
             self._check_accessed_members(cnode, accessed)
             
     def visit_function(self, node):
@@ -247,7 +231,7 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
             break
         # check if the method overload an attribute
         try:
-            overriden = klass.instance_attr(node.name)
+            overriden = klass.instance_attr(node.name)[0] # XXX
             while not isinstance(overriden, astng.Class):
                 overriden = overriden.parent.frame()
             self.add_message('E0202', args=overriden.name, node=node)
@@ -255,18 +239,22 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
             pass
                 
     def leave_function(self, node):
-        """check method arguments, overriding"""
-        # check first argument is self if this is actually a method
+        """on method node, check if this method couldn't be a function
+        
+        ignore class, static and abstract methods, initializer,
+        methods overriden from a parent class and any
+        kind of method defined in an interface for this warning
+        """
         if node.is_method():
             if node.argnames is not None:
                 self._first_attrs.pop()
-            # ignore class, static and abstract methods, initializer, and any
-            # kind of method defined in an interface for this warning
+            class_node = node.parent.frame()
             if (self._meth_could_be_func and node.type == 'method'
                 and node.name != '__init__'
-                and not node.is_abstract()
-                and node.parent.frame().type != 'interface'):
-                self.add_message('W0212', node=node)
+                and not (node.is_abstract() or
+                         overrides_a_method(class_node, node.name))
+                and class_node.type != 'interface'):
+                self.add_message('R0201', node=node)
                 
     def visit_getattr(self, node):
         """check if the name handle an access to a class member
@@ -280,16 +268,18 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
         """check if the name handle an access to a class member
         if so, register it
         """
-        if self._first_attrs and node.name == self._first_attrs[-1]:                
+        if self._first_attrs and (node.name == self._first_attrs[-1] or
+                                  not self._first_attrs[-1]):
             self._meth_could_be_func = False
                 
     def _check_accessed_members(self, node, accessed):
         """check that accessed members are defined"""
+        # XXX refactor, probably much simpler now that E0201 is in type checker
         for attr, nodes in accessed.items():
-            # is it a builtin attribute ?
-            if attr in ('__dict__', '__class__', '__doc__'):
-                # FIXME: old class object doesn't have __class__
-                continue
+##             # is it a builtin attribute ?
+##             if attr in ('__dict__', '__class__', '__doc__'):
+##                 # FIXME: old class object doesn't have __class__
+##                 continue
             # is it a class attribute ?
             try:
                 node.local_attr(attr)
@@ -304,30 +294,23 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
                 continue
             except StopIteration:
                 pass
-            # if we are running in zope mode, is it an acquired attribute ?
-            if self.config.zope and attr in self.config.acquired_members:
-                continue
             # is it an instance attribute ?
             try:
-                def_node = node.instance_attr(attr)
+                def_nodes = node.instance_attr(attr) # XXX
                 instance_attribute = True
             except astng.NotFoundError:
                 instance_attribute = False
             else:
-                # check that if the node is accessed in the same method as
-                # it's defined, it's accessed after the initial assigment
-                frame = def_node.frame()
-                lno = def_node.source_line()
-                for _node in nodes:
-                    if _node.frame() is frame and _node.lineno < lno:
-                        self.add_message('E0203', node=_node, args=(attr, lno))
-            for _node in nodes:
-                frame = _node.frame()
-                if frame.type == 'classmethod':
-                    if attr != '__name__':
-                        self.add_message('E0201', node=_node, args=attr)
-                elif not instance_attribute:
-                    self.add_message('E0201', node=_node, args=attr)
+                if len(def_nodes) == 1:
+                    def_node = def_nodes[0]
+                    # check that if the node is accessed in the same method as
+                    # it's defined, it's accessed after the initial assigment
+                    frame = def_node.frame()
+                    lno = def_node.source_line()
+                    for _node in nodes:
+                        if _node.frame() is frame and _node.lineno < lno:
+                            self.add_message('E0203', node=_node,
+                                             args=(attr, lno))
         
     def _check_first_arg_for_type(self, node, metaclass=0):
         """check the name of first argument, expect:
@@ -341,22 +324,22 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
         if node.argnames is None:
             return
         self._first_attrs.append(node.argnames and node.argnames[0])
-        # metaclass method
-        if metaclass:
-            if self._first_attrs[-1] != 'mcs':
-                self.add_message('E0214', node=node)
         # static method
-        elif node.type == 'staticmethod':
+        if node.type == 'staticmethod':
             if node.argnames and node.argnames[0] in ('self', 'cls', 'mcs'):
                 self.add_message('W0211', args=node.argnames[0], node=node)
             self._first_attrs[-1] = None
         # class / regular method with no args
         elif not node.argnames:
             self.add_message('E0211', node=node)
+        # metaclass method
+        elif metaclass:
+            if self._first_attrs[-1] != 'mcs':
+                self.add_message('C0203', node=node)
         # class method
         elif node.type == 'classmethod':
             if self._first_attrs[-1] != 'cls':
-                self.add_message('E0212', node=node)
+                self.add_message('C0202', node=node)
         # regular method without self as argument
         elif self._first_attrs[-1] != 'self':
             self.add_message('E0213', node=node)
@@ -367,10 +350,11 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
         """
         for method in node.methods():
             owner = method.parent.frame()
-            if owner is node:
+            if owner is node: 
                 continue
+            # owner is not this class, it must be a parent class
             # check that the ancestor's method is not abstract
-            if method.is_abstract(pass_is_abstract=0):
+            if method.is_abstract(pass_is_abstract=False):
                 self.add_message('W0223', node=node,
                                  args=(method.name, owner.name))
                     
@@ -378,69 +362,78 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
         """check that the given class node really implements declared
         interfaces
         """
-        def iface_handler(iface_node):
+        e0221_hack = [False]
+        def iface_handler(obj):
             """filter interface objects, it should be classes"""
-            try:
-                obj = iface_node.resolve_dotted(iface_node.as_string())
-                if not isinstance(obj, astng.Class):
-                    self.add_message('E0221', node=node,
-                                     args=(iface_node.as_string(),
-                                           obj.__class__.__name__))
-                else:
-                    yield obj
-            except:
-                self.add_message('F0203', node=node,
-                                 args=(iface_node.as_string()))
-                
+            if not isinstance(obj, astng.Class):
+                e0221_hack[0] = True
+                self.add_message('E0221', node=node,
+                                 args=(obj.as_string(),))
+                return False
+            return True
         ignore_iface_methods = self.config.ignore_iface_methods
-        for iface in node.interfaces(handler_func=iface_handler):
-            for imethod in iface.methods():
-                name = imethod.name
-                if name.startswith('_') or name in ignore_iface_methods:
-                    # don't check method begining with an underscore, usually
-                    # belonging to the interface implementation
-                    continue
-                # get class method astng
-                try:
-                    method = node_method(node, name)
-                except astng.NotFoundError:
-                    self.add_message('E0222', args=(name, iface.name),
-                                     node=node)
-                    continue
-                # ignore inherited methods
-                if method.parent.frame() is not node:
-                    continue
-                # check signature
-                self._check_signature(method, imethod,
-                                     '%s interface' % iface.name)
+        try:
+            for iface in node.interfaces(handler_func=iface_handler):
+                for imethod in iface.methods():
+                    name = imethod.name
+                    if name.startswith('_') or name in ignore_iface_methods:
+                        # don't check method begining with an underscore,
+                        # usually belonging to the interface implementation
+                        continue
+                    # get class method astng
+                    try:
+                        method = node_method(node, name)
+                    except astng.NotFoundError:
+                        self.add_message('E0222', args=(name, iface.name),
+                                         node=node)
+                        continue
+                    # ignore inherited methods
+                    if method.parent.frame() is not node:
+                        continue
+                    # check signature
+                    self._check_signature(method, imethod,
+                                         '%s interface' % iface.name)
+        except astng.InferenceError:
+            if e0221_hack[0]:
+                return
+            implements = astng.Instance(node).getattr('__implements__')[0]
+            assignment = implements.parent
+            assert isinstance(assignment, astng.Assign)
+            # assignment.expr can be a Name or a Tupe or whatever.
+            # Use as_string() for the message
+            # FIXME: in case of multiple interfaces, find which one could not
+            #        be resolved
+            self.add_message('F0220', node=implements,
+                             args=(node.name, assignment.expr.as_string()))
 
     def _check_init(self, node):
         """check that the __init__ method call super or ancestors'__init__
         method
         """
         klass_node = node.parent.frame()        
-        to_call, unresolved = self._ancestors_to_call(node, klass_node)
+        to_call = _ancestors_to_call(klass_node)
         for stmt in node.nodes_of_class(astng.CallFunc):
             expr = stmt.node
-            if not (isinstance(expr, astng.Name) or
-                    isinstance(expr, astng.Getattr)):
+            if not isinstance(expr, astng.Getattr) \
+                   or expr.attrname != '__init__':
                 continue
-            func_str = expr.as_string()
-            if func_str.endswith('.__init__'):
-                klass_name = '.'.join(func_str.split('.')[:-1])
+            # skip the test if using super
+            if isinstance(expr.expr, astng.CallFunc) and \
+               isinstance(expr.expr.node, astng.Name) and \
+               expr.expr.node.name == 'super':
+                return
+            try:
+                klass = expr.expr.infer().next()
                 try:
-                    del to_call[klass_name]
+                    del to_call[klass]
                 except KeyError:
-                    if unresolved.has_key(klass_name) \
-                           or klass_name in klass_node.basenames:
-                        continue
-                    if klass_name.startswith('super('):
-                        return
-                    self.add_message('W0233', node=expr, args=klass_name)
-        for klass_name in to_call.keys():
-            if klass_name == 'object':
+                    self.add_message('W0233', node=expr, args=klass.name)
+            except astng.InferenceError, ex:
                 continue
-            self.add_message('W0231', args=klass_name, node=node)
+        for klass in to_call.keys():
+            if klass.name == 'object':
+                continue
+            self.add_message('W0231', args=klass.name, node=node)
 
     def _check_signature(self, method1, method2, class_type):
         """check that the signature of the two given methods match
@@ -458,45 +451,20 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
             self.add_message('W0221', args=class_type, node=method1)
         elif len(method1.defaults) != len(method2.defaults):
             self.add_message('W0222', args=class_type, node=method1)
+
                         
-    def _ancestors_to_call(self, node, klass_node, method='__init__'):
-        """return two dictionarys :
-        
-        * one where keys are the list of base classes names with the given
-          method that should be called from the method node
-        * a second where keys are base classes names which could not have been
-          resolved correctly
-        """
-        to_call = {}
-        unresolved = {}
-        for base in klass_node.basenames:
-            parts = base.split('.')
-            cbase = parts[0]
-            try:
-                baseastng = klass_node.parent.resolve(cbase)
-            except (astng.ResolveError, astng.NotFoundError, AssertionError):
-                unresolved[base] = 1
-                self.add_message('F0203', node=node, args=cbase)
-                continue
-            for part in parts[1:]:
-                try:
-                    cbase = '%s.%s' % (cbase, part)
-                    baseastng = baseastng.resolve(part)
-                except astng.ResolveError:
-                    unresolved[base] = 1
-                    #import traceback
-                    #traceback.print_exc()
-                    self.add_message('F0203', node=node, args=cbase)
-                    break
-            else:
-                try:
-                    baseastng.local_attr(method)
-                    to_call[base] = 1
-                except AttributeError:
-                    self.add_message('F0204', node=baseastng, args=base)
-                except astng.NotFoundError:
-                    continue
-        return to_call, unresolved
+def _ancestors_to_call(klass_node, method='__init__'):
+    """return a dictionary where keys are the list of base classes providing
+    the queried method, and so that should/may be called from the method node
+    """
+    to_call = {}
+    for base_node in klass_node.ancestors(recurs=False):
+        try:
+            base_node.local_attr(method)
+            to_call[base_node] = 1
+        except astng.NotFoundError:
+            continue
+    return to_call
         
 
 def node_method(node, method_name):
