@@ -20,9 +20,9 @@
  FIXME: missing 13, 15, 16
 """
 
-__revision__ = "$Id: design_analysis.py,v 1.10 2005/11/02 09:21:46 syt Exp $"
+__revision__ = "$Id: design_analysis.py,v 1.11 2005-12-30 15:41:29 adim Exp $"
 
-from logilab.astng import Function
+from logilab.astng import Function, InferenceError
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
@@ -193,8 +193,12 @@ class MisdesignChecker(BaseChecker):
                 if parent.name == 'Interface':
                     continue
                 self._used_ifaces[parent] = 1
-        for iface in node.interfaces():
-            self._used_ifaces[iface] = 1
+        try:
+            for iface in node.interfaces():
+                self._used_ifaces[iface] = 1
+        except InferenceError:
+            # XXX log ? 
+            pass
         for parent in node.ancestors():
             try:
                 self._used_abstracts[parent] += 1
