@@ -22,7 +22,7 @@ http://www.python.org/doc/essays/styleguide.html
 Some parts of the process_token method is based from The Tab Nanny std module.
 """
 
-__revision__ = "$Id: format.py,v 1.49 2005/11/02 09:21:46 syt Exp $"
+__revision__ = "$Id: format.py,v 1.51 2006-03-14 15:08:10 syt Exp $"
 
 import re
 import tokenize
@@ -62,7 +62,8 @@ MSGS = {
               'Used when a comma (",") is not followed by a space.'),
     
     'W0331': ('Use of the <> operator',
-              'Used when the deprecated "<>" operator is used instead of "!=".'),
+              'Used when the deprecated "<>" operator is used instead \
+              of "!=".'),
     'W0332': ('Use l as long integer identifier',
               'Used when a lower case "l" is used to mark a long integer. You \
 should use a upper case "L" since the letter "l" looks too much like the digit \
