@@ -16,7 +16,7 @@
 """imports checkers for Python code
 """
 
-__revision__ = "$Id: imports.py,v 1.43 2005/11/02 09:21:47 syt Exp $"
+__revision__ = "$Id: imports.py,v 1.46 2005/11/10 17:26:27 syt Exp $"
 
 from logilab.common import get_cycles
 from logilab.common.modutils import is_standard_module, is_relative, \
@@ -145,7 +145,7 @@ MSGS = {
               'Used when a cyclic import between two or more modules is \
               detected.'),
     
-    'W0401': ('Wildcard import',
+    'W0401': ('Wildcard import %s',
               'Used when `from module import *` is detected.'),
     'W0402': ('Uses of a deprecated module %r',
               'Used a module marked as deprecated is imported.'),
@@ -156,6 +156,10 @@ MSGS = {
               'Used when a module is reimported multiple times.'),
     'W0406': ('Module import itself',
               'Used when a module is importing itself.'),
+    
+    'W0410': ('__future__ import is not the first non docstring statement',
+              'Python 2.5 and greater require __future__ import to be the \
+              first non docstring statement in the module.'),
     }
 
 class ImportsChecker(BaseChecker):
@@ -240,11 +244,15 @@ given file (report R0402 must not be disabled)'}
     def visit_from(self, node):
         """triggered when an import statement is seen"""
         basename = node.modname
+        if basename == '__future__':
+            # check this is the first non docstring statement in the module
+            if node.previous_sibling():
+                self.add_message('W0410', node=node)                    
         self._check_deprecated(node, basename)
         relative = self._check_relative(node, basename)
         for name, _ in node.names:
             if name == '*':
-                self.add_message('W0401', node=node)
+                self.add_message('W0401', args=basename, node=node)
                 continue
             # handle reimport
             self._check_reimport(node, name, basename)
@@ -252,9 +260,11 @@ given file (report R0402 must not be disabled)'}
             fullname = '%s.%s' % (basename, name)
             if fullname.find('.') > -1:
                 try:
+                    # XXXFIXME: don't use get_module_part which doesn't take
+                    # care of package precedence
                     fullname = get_module_part(fullname,
                                                context_file=node.root().file)
-                except Exception, ex:
+                except ImportError, ex:
                     self.add_message('F0401', args=(fullname, ex), node=node)
                     continue            
             self._imported_module(node, fullname, relative)
