@@ -10,13 +10,13 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-""" Copyright (c) 2002-2005 LOGILAB S.A. (Paris, FRANCE).
+""" Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 
 Raw metrics checker
 """
 
-__revision__ = "$Id: raw_metrics.py,v 1.14 2005/04/15 10:40:11 syt Exp $"
+__revision__ = "$Id: raw_metrics.py,v 1.15 2005/11/02 09:21:47 syt Exp $"
 
 import tokenize
 
@@ -30,6 +30,28 @@ from pylint.interfaces import IRawChecker
 from pylint.checkers import BaseRawChecker, EmptyReport
 from pylint.reporters import diff_string
 
+
+def report_raw_stats(sect, stats, old_stats):
+    """calculate percentage of code / doc / comment / empty
+    """
+    total_lines = stats['total_lines']
+    if not total_lines:
+        raise EmptyReport()
+    sect.description = '%s lines have been analyzed' % total_lines
+    lines = ('type', 'number', '%', 'previous', 'difference')
+    for node_type in ('code', 'docstring', 'comment', 'empty'):
+        key = node_type + '_lines'
+        total = stats[key]
+        percent = float(total * 100) / total_lines
+        old = old_stats.get(key, None)
+        if old is not None:
+            diff_str = diff_string(old, total)
+        else:
+            old, diff_str = 'NC', 'NC'
+        lines += (node_type, str(total), '%.2f' % percent,
+                  str(old), diff_str)
+    sect.append(Table(children=lines, cols=5, rheaders=1))
+
  
 class RawMetricsChecker(BaseRawChecker):
     """does not check anything but gives some raw metrics :                    
@@ -48,12 +70,12 @@ class RawMetricsChecker(BaseRawChecker):
     options = ( )
     # messages
     msgs = {}
+    # reports
+    reports = ( ('R0701', 'Raw metrics', report_raw_stats), )
     
     def __init__(self, linter):
         BaseRawChecker.__init__(self, linter)
         self.stats = None
-        self.reports = (('R0701', 'Raw metrics', self.report_raw_stats),
-                        )
         
     def open(self):
         """init statistics"""
@@ -70,27 +92,6 @@ class RawMetricsChecker(BaseRawChecker):
             self.stats['total_lines'] += lines_number
             self.stats[line_type] += lines_number
 
-    def report_raw_stats(self, sect, stats, old_stats):
-        """calculate percentage of code / doc / comment / empty
-        """
-        total_lines = stats['total_lines']
-        if not total_lines:
-            raise EmptyReport()
-        sect.description = '%s lines have been analyzed' % total_lines
-        lines = ('type', 'number', '%', 'previous', 'difference')
-        for node_type in ('code', 'docstring', 'comment', 'empty'):
-            key = node_type + '_lines'
-            total = stats[key]
-            percent = float(total * 100) / total_lines
-            old = old_stats.get(key, None)
-            if old is not None:
-                diff_str = diff_string(old, total)
-            else:
-                old, diff_str = 'NC', 'NC'
-            lines += (node_type, str(total), '%.2f' % percent,
-                      str(old), diff_str)
-        sect.append(Table(children=lines, cols=5, rheaders=1))
-                    
         
 JUNK = (tokenize.NL, tokenize.INDENT, tokenize.NEWLINE, tokenize.ENDMARKER)
 
