@@ -18,7 +18,7 @@
 """
 from __future__ import generators
 
-__revision__ = '$Id: similar.py,v 1.12 2005/11/02 09:21:48 syt Exp $'
+__revision__ = '$Id: similar.py,v 1.14 2006-03-29 08:24:32 syt Exp $'
 
 import sys
 
@@ -32,16 +32,18 @@ from pylint.checkers import BaseChecker, table_lines_from_stats
 class Similar:
     """finds copy-pasted lines of code in a project"""
     
-    def __init__(self, min_lines=4, ignore_comments=False):
+    def __init__(self, min_lines=4, ignore_comments=False, ignore_docstrings=False):
         self.min_lines = min_lines
         self.ignore_comments = ignore_comments
+        self.ignore_docstrings = ignore_docstrings
         self.linesets = []
 
     def append_stream(self, streamid, stream):
         """append a file to search for similarities"""
         self.linesets.append(LineSet(streamid,
                                      stream.readlines(),
-                                     self.ignore_comments))
+                                     self.ignore_comments,
+                                     self.ignore_docstrings))
         
     def run(self):
         """start looking for similarities and display results on stdout"""
@@ -123,23 +125,34 @@ class Similar:
                 for sim in self._find_common(lineset, lineset2):
                     yield sim
 
+def stripped_lines(lines, ignore_comments, ignore_docstrings):
+    strippedlines = []
+    docstring = None
+    for line in lines:
+        line = line.strip()
+        if ignore_docstrings:
+            if not docstring and \
+                   (line.startswith('"""') or line.startswith("'''")):
+                docstring = line[:3]
+                line = line[3:]
+            if docstring:
+                if line.endswith(docstring):
+                    docstring = None
+                line = ''
+        # XXX cut when a line begins with code but end with a comment
+        if ignore_comments and line.startswith('#'):
+                line = ''
+        strippedlines.append(line)
+    return strippedlines
 
 class LineSet:
     """Holds and indexes all the lines of a single source file"""
-    def __init__(self, name, lines, ignore_comments=False):
+    def __init__(self, name, lines, ignore_comments=False, ignore_docstrings=False):
         self.name = name
         self._real_lines = lines
-        if ignore_comments:
-            self._stripped_lines = []
-            for line in lines:
-                line = line.strip()
-                if line.startswith('#'):
-                    line = ''
-                self._stripped_lines.append(line)
-        else:
-            self._stripped_lines = [line.strip() for line in lines]
+        self._stripped_lines = stripped_lines(lines, ignore_comments, ignore_docstrings)
         self._index = self._mk_index()
-        
+            
     def __str__(self):
         return '<Lineset for %s>' % self.name
 
@@ -217,13 +230,18 @@ class SimilarChecker(BaseChecker, Similar):
                 {'default' : True, 'type' : 'yn', 'metavar' : '<y or n>',
                  'help': 'Ignore comments when computing similarities.'}
                 ),
+               ('ignore-docstrings',
+                {'default' : True, 'type' : 'yn', 'metavar' : '<y or n>',
+                 'help': 'Ignore docstrings when computing similarities.'}
+                ),
                )
     # reports
     reports = ( ('R0801', 'Duplication', report_similarities), )
     
     def __init__(self, linter=None):
         BaseChecker.__init__(self, linter)
-        Similar.__init__(self, min_lines=4, ignore_comments=True)
+        Similar.__init__(self, min_lines=4,
+                         ignore_comments=True, ignore_docstrings=True)
         self.stats = None
 
     def set_option(self, opt_name, value, action=None, opt_dict=None):
@@ -236,6 +254,8 @@ class SimilarChecker(BaseChecker, Similar):
             self.min_lines = self.config.min_similarity_lines
         elif opt_name == 'ignore-comments':
             self.ignore_comments = self.config.ignore_comments
+        elif opt_name == 'ignore-docstrings':
+            self.ignore_docstrings = self.config.ignore_docstrings
         
     def open(self):
         """init the checkers: reset linesets and statistics information"""
