@@ -18,10 +18,10 @@
 """some functions that may be usefull for various checkers
 """
 
-__revision__ = '$Id: utils.py,v 1.12 2005/11/02 09:21:48 syt Exp $'
+__revision__ = '$Id: utils.py,v 1.16 2006-03-03 09:25:34 syt Exp $'
 
 from logilab import astng
-
+from logilab.astng.utils import are_exclusive
 try:
     # python >= 2.4
     COMP_NODE_TYPES = (astng.ListComp, astng.GenExpr)
@@ -118,14 +118,6 @@ def is_ancestor_name(frame, node):
             return True
     return False
 
-def is_parent(pnode, node):
-    """return True if pnode is a parent of node"""
-    while node:
-        node = node.parent
-        if node is pnode:
-            return True
-    return False
-
 def assign_parent(node):
     """return the higher parent which is not an AssName, AssTuple or AssList
     node
@@ -136,44 +128,17 @@ def assign_parent(node):
         node = node.parent
     return node
 
-def are_exclusive(stmt1, stmt2):
-    """return true if the two given statement are mutually exclusive
+def overrides_an_abstract_method(class_node, name):
+    """return True if pnode is a parent of node"""
+    for ancestor in class_node.ancestors():
+        if name in ancestor and isinstance(ancestor[name], astng.Function) and \
+               ancestor[name].is_abstract(pass_is_abstract=False):
+            return True
+    return False
 
-    algorithm :
-     1) index stmt1's parents
-     2) climb among stmt2's parents until we find a common parent
-     3) if the common parent is a If or TryExcept statement, look if nodes are
-        in exclusive branchs
-    """
-    # index stmt1's parents
-    stmt1_parents = {}
-    children = {}
-    node = stmt1.parent
-    previous = stmt1
-    while node:
-        stmt1_parents[node] = 1
-        children[node] = previous
-        previous = node
-        node = node.parent
-    # climb among stmt2's parents until we find a common parent
-    node = stmt2.parent
-    previous = stmt2
-    result = 0
-    while node:
-        if stmt1_parents.has_key(node):
-            # if the common parent is a If or TryExcept statement, look if
-            # nodes are in exclusive branchs
-            if isinstance(node, astng.If):
-                if previous != children[node]:
-                    result = 1
-            elif isinstance(node, astng.TryExcept):
-                stmt1_previous = children[node]
-                if previous != stmt1_previous and not (
-                    (previous is node.body and stmt1_previous is node.else_) or
-                    (stmt1_previous is node.body and previous is node.else_)):
-                    result = 1
-                
-            break
-        previous = node
-        node = node.parent
-    return result
+def overrides_a_method(class_node, name):
+    """return True if <name> is a method overriden from an ancestor"""
+    for ancestor in class_node.ancestors():
+        if name in ancestor and isinstance(ancestor[name], astng.Function):
+            return True
+    return False
