@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -16,37 +16,45 @@
 """variables checkers for Python code
 """
 
-__revision__ = "$Id: variables.py,v 1.55 2005/11/07 13:39:57 syt Exp $"
+__revision__ = "$Id: variables.py,v 1.69 2006-04-19 09:17:40 syt Exp $"
 
 from copy import copy
 
+from logilab.common.compat import enumerate
 from logilab import astng
+from logilab.astng.lookup import builtin_lookup
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import is_error, is_builtin, is_func_default, \
-     is_ancestor_name, is_parent, assign_parent, are_exclusive, \
-     is_defined_before, FOR_NODE_TYPES
+     is_ancestor_name, assign_parent, are_exclusive, \
+     is_defined_before #, is_parent, FOR_NODE_TYPES
 
 
     
 MSGS = {
     'E0601': ('Using variable %r before assignment',
-              'Used when a local variable is accessed before it\'s assignment.'),
+              'Used when a local variable is accessed before it\'s \
+              assignment.'),
     'E0602': ('Undefined variable %r',
               'Used when an undefined variable is accessed.'),
 
     'E0611': ('No name %r in module %r',
               'Used when a name cannot be found in a module.'),
     
-    'W0601': ('Undefined global %r',
+    'W0601': ('Global variable %r undefined at the module level',
               'Used when a variable is defined through the "global" statement \
-              but the variable is not defined in the static global scope.'),
-#    'W0602': ('Using possibly undefined loop variable %r',
-#              'Used when an loop variable (i.e. defined by a for loop or \
-#              a list comprehension or a generator expression) is used outside \
-#              the loop.'),
-    
+              but the variable is not defined in the module scope.'),
+    'W0602': ('Using global for %r but no assigment is done',
+              'Used when a variable is defined through the "global" statement \
+              but no assigment to this variable is done.'),
+    'W0603': ('Using the global statement', # W0121
+              'Used when you use the "global" statement to update a global \
+              variable. PyLint just try to discourage this \
+              usage. That doesn\'t mean you can not use it !'),
+    'W0604': ('Using the global statement at the module level', # W0103
+              'Used when you use the "global" statement at the module level \
+              since it has no effect'),
     'W0611': ('Unused import %s',
               'Used when an imported module or variable is not used.'),
     'W0612': ('Unused variable %r',
@@ -59,6 +67,11 @@ MSGS = {
               scope.'),
     'W0622': ('Redefining built-in %r',
               'Used when a variable or function override a built-in.'),
+
+    'W0631': ('Using possibly undefined loop variable %r',
+              'Used when an loop variable (i.e. defined by a for loop or \
+              a list comprehension or a generator expression) is used outside \
+              the loop.'),
     }
 
 class VariablesChecker(BaseChecker):
@@ -79,11 +92,11 @@ class VariablesChecker(BaseChecker):
                 {'default': 0, 'type' : 'yn', 'metavar' : '<y_or_n>',
                  'help' : 'Tells wether we should check for unused import in \
 __init__ files.'}),
-               ("dummy-variables",
-                {'default': ('_', 'dummy'), 'type' : 'csv',
-                 'metavar' : '<comma separated list>',
-                 'help' : 'List of variable names used for dummy variables \
-(i.e. not used).'}),
+               ("dummy-variables-rgx",
+                {'default': ('_|dummy'), 
+                 'type' :'regexp', 'metavar' : '<regexp>',
+                 'help' : 'A regular expression matching names used \
+                 for dummy variables (i.e. not used).'}),
                ("additional-builtins",
                 {'default': (), 'type' : 'csv',
                  'metavar' : '<comma separated list>',
@@ -149,14 +162,6 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         # do not check for not used locals here
         self._to_consume.pop()
 
-    def _assign_name(self, name, node):
-        try:
-            names_dict = self._vars[-1]
-        except IndexError:
-            return
-        if not name in names_dict:
-            names_dict[name] = node
-        
     def visit_function(self, node):
         """visit function: update consumption analysis variable and check locals
         """
@@ -182,10 +187,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             return
         if is_error(node):
             return
-        authorized = self.config.dummy_variables
+        authorized_rgx = self.config.dummy_variables_rgx
         for name, stmts in not_consumed.items():
             # ignore some special names specified by user configuration
-            if name in authorized:
+            if authorized_rgx.match(name):
                 continue
             # ignore names imported by the global statement
             # FIXME: should only ignore them if it's assigned latter
@@ -207,11 +212,60 @@ builtins. Remember that you should avoid to define new builtins when possible.'
 
     def visit_global(self, node):
         """check names imported exists in the global scope"""
-        globs = node.root().globals
+        frame = node.frame()
+        if isinstance(frame, astng.Module):
+            self.add_message('W0604', node=node)
+            return        
+        module = frame.root()
+        default_message = True
         for name in node.names:
-            if not globs.has_key(name):
+            try:
+                assign_nodes = module.getattr(name)
+            except astng.NotFoundError:
+                # unassigned global, skip
+                assign_nodes = []
+            for anode in assign_nodes:
+                if anode.frame() is frame:
+                    # same scope level assigment
+                    break
+            else:
+                # global but no assigment
+                self.add_message('W0602', args=name, node=node)
+                default_message = False
+            if not assign_nodes:
+                continue
+            for anode in assign_nodes:
+                if anode.frame() is module:
+                    # module level assigment
+                    break
+            else:
+                # global undefined at the module scope
                 self.add_message('W0601', args=name, node=node)
-                
+                default_message = False
+        if default_message:
+            self.add_message('W0603', node=node) 
+
+    def _loopvar_name(self, node, name):
+        # filter variables according to node's scope
+        astmts = [stmt for stmt in node.lookup(name)[1]
+                  if hasattr(stmt, 'ass_type') and
+                  not stmt.statement().parent_of(node)]
+        # filter variables according their respective scope
+        if not astmts or astmts[0].statement().parent_of(node):
+            _astmts = []
+        else:
+            _astmts = astmts[:1]
+        for i, stmt in enumerate(astmts[1:]):
+            if astmts[i].statement().parent_of(stmt):
+                continue
+            _astmts.append(stmt)
+        astmts = _astmts
+        if len(astmts) == 1:
+            ass = astmts[0].ass_type()
+            if isinstance(ass, (astng.For, astng.ListCompFor, astng.GenExpr)) \
+                   and not ass.statement() is node.statement():
+                self.add_message('W0631', args=name, node=node)
+    
     def visit_name(self, node):
         """check that a name is defined if the current scope and doesn't
         redefine a built-in
@@ -235,43 +289,43 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             # names
             if scope_type == 'class' and i != start_index:
                 continue
-## XXXFIXME
-##             # the name has already been consumed, only check it's not a loop
-##             # variable used outside the loop
+            # the name has already been consumed, only check it's not a loop
+            # variable used outside the loop
             if consumed.has_key(name):
-##                 def_node = assign_parent(node.nearest(consumed[name]))
-##                 if isinstance(def_node, FOR_NODE_TYPES) \
-##                    and not is_parent(def_node.statement(), node):
-##                     self.add_message('W0602', args=name, node=node)
+                self._loopvar_name(node, name)
                 break
             # mark the name as consumed if it's defined in this scope
             # (ie no KeyError is raise by "to_consume[name]"
             try:
                 consumed[name] = to_consume[name]
-## XXXFIXME
-##                 def_node = assign_parent(node.nearest(to_consume[name]))
-##                 if def_node is not None:
-##                     def_stmt = def_node.statement()
-##                     if isinstance(def_node, FOR_NODE_TYPES) \
-##                        and not is_parent(def_stmt, node):
-##                         self.add_message('W0602', args=name, node=node)                        
                 # checks for use before assigment
                 # FIXME: the last condition should just check attribute access
                 # is protected by a try: except NameError: (similar to #9219)
-                def_node = assign_parent(to_consume[name][0])
-                if def_node is not None:
-                    def_stmt = def_node.statement()
-                    root = def_stmt.root()
-##                     print stmt
-##                     print def_stmt
-##                     print frame is root, name, not name in frame
-                    if (frame is def_stmt.frame()
-                          #and (frame is root or not name in frame)
-                          and stmt.source_line() <= def_stmt.source_line()
-                          and not is_defined_before(node)
-                          and not are_exclusive(stmt, def_stmt)):
-                        self.add_message('E0601', args=name, node=node)                
+                defnode = assign_parent(to_consume[name][0])
+                if defnode is not None:
+                    defstmt = defnode.statement()
+                    defframe = defstmt.frame()
+                    maybee0601 = True
+                    if not frame is defframe:
+                        maybee0601 = False
+                    elif defframe.parent is None:
+                        # we are at the module level, check the name is not
+                        # defined in builtins
+                        if builtin_lookup(name)[1]:
+                            maybee0601 = False
+                    else:
+                        # we are in a local scope, check the name is not
+                        # defined in global or builtin scope
+                        if defframe.root().lookup(name)[1]:
+                            maybee0601 = False
+                    if (maybee0601
+                        and stmt.source_line() <= defstmt.source_line()
+                        and not is_defined_before(node)
+                        and not are_exclusive(stmt, defstmt)):
+                        self.add_message('E0601', args=name, node=node)
                 del to_consume[name]
+                # check it's not a loop variable used outside the loop
+                self._loopvar_name(node, name)
                 break
             except KeyError:
                 continue
@@ -279,27 +333,23 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             # we have not found the name, if it isn't a builtin, that's an
             # undefined name !
             if not self._is_builtin(name):
-                self.add_message('E0602', args=name, node=stmt)
+                self.add_message('E0602', args=name, node=node)
                 
     def visit_import(self, node):
         """check modules attribute accesses"""
-        for name, asname in node.names:
+        for name, _ in node.names:
             name_parts = name.split('.')
             try:
-                module = node.self_resolve(name_parts[0], None, asname=False)
+                module = node.infer(name_parts[0], asname=False).next()
             except astng.ResolveError:
                 continue
-            except TypeError:
-                # XXX for backward compat (astng < 0.13.1), remove latter
-                continue
             self._check_module_attrs(node, module, name_parts[1:])
                                     
     def visit_from(self, node):
         """check modules attribute accesses"""
         name_parts = node.modname.split('.')
         try:
-            module = self.linter.manager.astng_from_module_name(
-                name_parts[0], context_file=node.root().file)
+            module = node.root().import_module(name_parts[0])
         except KeyboardInterrupt:
             raise
         except:
@@ -307,35 +357,35 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         module = self._check_module_attrs(node, module, name_parts[1:])
         if not module:
             return
-        for name, asname in node.names:
+        for name, _ in node.names:
             if name == '*':
                 continue
             self._check_module_attrs(node, module, name.split('.'))
 
-    def leave_getattr(self, node):
-        """check modules attribute accesses
+##     def leave_getattr(self, node):
+##         """check modules attribute accesses
         
-        this function is a "leave_" because when parsing 'a.b.c'
-        we want to check the innermost expression first.
-        """
-        if isinstance(node.expr, astng.Name):
-            try:
-                module = node.resolve(node.expr.name)
-            except astng.ResolveError:
-                return
-            if not isinstance(module, astng.Module):
-                # Not a module, don't check
-                return
-        elif self._checking_mod_attr is not None:
-            module = self._checking_mod_attr
-        else:
-            return
-        self._checking_mod_attr = self._check_module_attrs(node, module,
-                                                           [node.attrname])
+##         this function is a "leave_" because when parsing 'a.b.c'
+##         we want to check the innermost expression first.
+##         """
+##         if isinstance(node.expr, astng.Name):
+##             try:
+##                 module = node.expr.infer().next()
+##             except astng.InferenceError:
+##                 return
+##             if not isinstance(module, astng.Module):
+##                 # Not a module, don't check
+##                 return
+##         elif self._checking_mod_attr is not None:
+##             module = self._checking_mod_attr
+##         else:
+##             return
+##         self._checking_mod_attr = self._check_module_attrs(node, module,
+##                                                            [node.attrname])
 
-    def leave_default(self, node):
-        """by default, reset the _checking_mod_attr attribute"""
-        self._checking_mod_attr = None
+##     def leave_default(self, node):
+##         """by default, reset the _checking_mod_attr attribute"""
+##         self._checking_mod_attr = None
         
     def _check_module_attrs(self, node, module, module_names):
         """check that module_names (list of string) are accessible through the
@@ -345,22 +395,22 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         assert isinstance(module, astng.Module), module
         while module_names:
             name = module_names.pop(0)
-            # module / package locals precedence
-            if name in module:
-                module = module[name]
-                continue
             if name == '__dict__':
+                module = None
                 break
             try:
-                module = module.resolve(name)
-            except astng.ResolveError:
-                self.add_message('E0611', args=(name, module.name),
-                                 node=node)
+                module = module.getattr(name)[0].infer().next()
+            except astng.NotFoundError:
+                self.add_message('E0611', args=(name, module.name), node=node)
+                return None
+            except astng.InferenceError:
                 return None
         if module_names:
-            # FIXME: other message if name is not the latest part of module_names ?
-            self.add_message('E0611', args=('.'.join(module_names), module.name),
-                             node=node)
+            # FIXME: other message if name is not the latest part of
+            # module_names ?
+            modname = module and module.name or '__dict__'
+            self.add_message('E0611', node=node,
+                             args=('.'.join(module_names), modname))
             return None
         if isinstance(module, astng.Module):
             return module
