@@ -1,4 +1,4 @@
-from logilab.common import astng
+from logilab import astng
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
@@ -27,7 +27,7 @@ class MyASTNGChecker(BaseChecker):
                 and node.node.expr.name == 'properties'
                 and node.node.attrname == 'create'):
             return
-        in_class = node.get_frame()
+        in_class = node.frame()
         for param in node.args:
             in_class.locals[param.name] = node
 
