@@ -1,5 +1,5 @@
-# Copyright (c) 2003-2005 Sylvain Thenault (thenault@gmail.com).
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2006 Sylvain Thenault (thenault@gmail.com).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -27,9 +27,7 @@
   Display help messages about given message identifiers and exit.
 """
 
-__revision__ = "$Id: lint.py,v 1.103 2005/11/02 17:16:50 syt Exp $"
-
-from __future__ import nested_scopes
+__revision__ = "$Id: lint.py,v 1.115 2006-04-19 09:17:40 syt Exp $"
 
 # import this first to avoid further builtins pollution possibilities
 from pylint.checkers import utils
@@ -43,13 +41,16 @@ from os.path import dirname, basename, splitext, exists, isdir, join, normpath
 from logilab.common.configuration import OptionsManagerMixIn, \
      check_yn, check_csv
 from logilab.common.modutils import modpath_from_file, get_module_files, \
-     file_from_modpath
+     file_from_modpath, load_module_from_name
 from logilab.common.interface import implements
 from logilab.common.textutils import get_csv
 from logilab.common.fileutils import norm_open
 from logilab.common.ureports import Table, Text
 from logilab.common.compat import enumerate
+from logilab.common.__pkginfo__ import version as common_version
+
 from logilab.astng import ASTNGManager
+from logilab.astng.__pkginfo__ import version as astng_version
 
 from pylint.utils import UnknownMessage, MessagesHandlerMixIn, \
      ReportsHandlerMixIn, MSG_TYPES, sort_checkers
@@ -64,7 +65,7 @@ from pylint import config
 from pylint.__pkginfo__ import version
 
 
-OPTION_RGX = re.compile('#*\s*pylint:(.*)')
+OPTION_RGX = re.compile('\s*#*\s*pylint:(.*)')
 REPORTER_OPT_MAP = {'html': HTMLReporter,
                     'parseable': TextReporter2,
                     'color': ColorizedTextReporter}
@@ -130,6 +131,7 @@ class PyLinter(OptionsManagerMixIn, MessagesHandlerMixIn, ReportsHandlerMixIn,
                  'dest' : 'black_list', 'default' : ('CVS',),
                  'help' : 'Add <file or directory> to the black list. It \
 should be a base name, not a path. You may set this option multiple times.'}),
+               
                ('persistent',
                 {'default': 1, 'type' : 'yn', 'metavar' : '<y_or_n>',
                  'help' : 'Pickle collected data for later comparisons.'}),
@@ -138,6 +140,11 @@ should be a base name, not a path. You may set this option multiple times.'}),
                 {'default': 500, 'type' : 'int', 'metavar': '<size>',
                  'help' : 'Set the cache size for astng objects.'}),
                
+               ('load-plugins',
+                {'type' : 'csv', 'metavar' : '<modules>', 'default' : (),
+                 'help' : 'List of plugins (as comma separated values of \
+python modules names) to load, usually to register additional checkers.'}),
+               
                ('reports',
                 {'default': 1, 'type' : 'yn', 'metavar' : '<y_or_n>',
                  'short': 'r',
@@ -252,8 +259,8 @@ This is used by the global evaluation report (R0004).'}),
             'disable-msg': self.disable_message,
             'enable-msg-cat': self.enable_message_category,
             'disable-msg-cat': self.disable_message_category}
-        full_version = '%%prog %s\nPython %s' % (version,
-                                                 sys.version)
+        full_version = '%%prog %s, \nastng %s, common %s\nPython %s' % (
+            version, astng_version, common_version, sys.version)
         OptionsManagerMixIn.__init__(self, usage=__doc__,
                                      version=full_version,
                                      config_file=pylintrc or config.PYLINTRC)
@@ -271,6 +278,18 @@ This is used by the global evaluation report (R0004).'}),
                          self.report_evaluation),
                         )
         self.register_checker(self)
+        self._dynamic_plugins = []
+        
+    def load_plugin_modules(self, modnames):
+        """take a list of module names which are pylint plugins and load
+        and register them
+        """
+        for modname in modnames:
+            if modname in self._dynamic_plugins:
+                continue
+            self._dynamic_plugins.append(modname)
+            module = load_module_from_name(modname)
+            module.register(self)
 
     def set_reporter(self, reporter):
         """set the reporter used to display messages and reports"""
@@ -292,6 +311,8 @@ This is used by the global evaluation report (R0004).'}),
                     meth(value)
         elif opt_name == 'cache-size':
             self.manager.set_cache_size(int(value))
+#        elif opt_name = 'load-plugins':
+#            self.load_plugin_modules(get_csv(value))
         elif opt_name in REPORTER_OPT_MAP and check_yn(None, opt_name, value):
             self.set_reporter(REPORTER_OPT_MAP[opt_name]())
         BaseRawChecker.set_option(self, opt_name, value, action, opt_dict)
@@ -327,20 +348,39 @@ This checker also defines the following reports:
         for checker in self._checkers.keys():
             checker.enable(False)
 
-
+    def disable_noerror_checkers(self):
+        """disable all checkers without error messages, and the
+        'miscellaneous' checker which can be safely deactivated in debug
+        mode
+        """
+        for checker in self._checkers.keys():
+            if checker.name == 'miscellaneous':
+                checker.enable(False)
+                continue
+            for msgid in getattr(checker, 'msgs', {}).keys():
+                if msgid[0] == 'E':
+                    checker.enable(True)
+                    break
+            else:
+                checker.enable(False)
+                
+    # block level option handling #############################################
+    #
+    # see func_block_disable_msg.py test case for expected behaviour
+    
     def process_tokens(self, tokens):
-        """process tokens from the current module to search for module level
-        options
+        """process tokens from the current module to search for module/block
+        level options
         """
         comment = tokenize.COMMENT
         newline = tokenize.NEWLINE
         #line_num = 0
         for (tok_type, _, start, _, line) in tokens:
             if tok_type not in (comment, newline):
-                break
+                continue
             #if start[0] == line_num:
             #    continue
-            match = OPTION_RGX.match(line)
+            match = OPTION_RGX.search(line)
             if match is None:
                 continue
             try:
@@ -353,14 +393,38 @@ This checker also defines the following reports:
             #line_num = start[0]
             if opt in self._options_methods and not opt.endswith('-report'):
                 meth = self._options_methods[opt]
-                for msg_id in get_csv(value):
+                for msgid in get_csv(value):
                     try:
-                        meth(msg_id, 'module', start[0])
+                        meth(msgid, 'module', start[0])
                     except UnknownMessage:
-                        self.add_message('E0012', args=msg_id, line=start[0])
+                        self.add_message('E0012', args=msgid, line=start[0])
             else:
                 self.add_message('E0011', args=opt, line=start[0])
-        
+    
+    def collect_block_lines(self, node, msg_state):
+        """walk ast to collect block level options line numbers"""
+        # recurse on children (depth first)
+        for child in node.getChildNodes():
+            self.collect_block_lines(child, msg_state)            
+        for msgid, lines in msg_state.items():
+            #if msg in self._module_msgs_state:
+            #    continue
+            for lineno, state in lines.items():
+                first = node.source_line()
+                last = node.last_source_line()
+                if lineno >= first and lineno <= last:
+                    # set state for all lines for this block
+                    first, last = node.block_range(lineno)
+                    for line in xrange(first, last+1):
+                        # do not override existing entries
+                        if not line in self._module_msgs_state.get(msgid, ()):
+                            try:
+                                self._module_msgs_state[msgid][line] = state
+                            except KeyError:
+                                self._module_msgs_state[msgid] = {line: state}
+                    del lines[lineno]
+                    
+    
     # code checking methods ###################################################
 
     def check(self, files_or_modules):
@@ -383,7 +447,7 @@ This checker also defines the following reports:
                 # this is a file or a directory
                 try:
                     modname = '.'.join(modpath_from_file(something))
-                except Exception:
+                except ImportError:
                     modname = splitext(basename(something))[0]
                 if isdir(something):
                     filepath = join(something, '__init__.py')
@@ -398,7 +462,7 @@ This checker also defines the following reports:
                         self.set_current_module(modname)
                         self.add_message('F0003', args=modname)
                         continue
-                except Exception, ex:
+                except ImportError, ex:
                     #if __debug__:
                     #    import traceback
                     #    traceback.print_exc()
@@ -486,8 +550,16 @@ This checker also defines the following reports:
         else:
             #assert astng.file.endswith('.py')
             stream = norm_open(astng.file)
+            # invoke IRawChecker interface on self to fetch module/block
+            # level options
+            self.process_module(stream)
+            # walk ast to collect line numbers
+            orig_state = self._module_msgs_state.copy()
+            self._module_msgs_state = {}
+            self.collect_block_lines(astng, orig_state)
+
             for checker in checkers:
-                if implements(checker, IRawChecker):
+                if implements(checker, IRawChecker) and checker is not self:
                     stream.seek(0)
                     checker.process_module(stream)
         # generate events to astng checkers
@@ -632,6 +704,23 @@ except AttributeError:
     __builtins__['_'] = str
 
 
+def preprocess_options(args, search_for):
+    """look for some options (keys of <search_for>) which have to be processed
+    before others
+    
+    values of <search_for> are callback functions to call when the option is
+    found
+    """
+    for i, arg in enumerate(args):
+        for option in search_for:
+            ##print arg, option
+            if arg.startswith('--%s=' % option):
+                search_for[option](option, arg[len(option)+3:])
+                del args[i]
+            elif arg == '--%s' % option:
+                search_for[option](option, args[i + 1])
+                del args[i:i+2]
+    
 class Run:
     """helper class to use as main for pylint :
     
@@ -639,16 +728,10 @@ class Run:
     """
     
     def __init__(self, args, reporter=None, quiet=0):
-        rcfile = None
-        for i, arg in enumerate(args):
-            if arg.startswith('--rcfile='):
-                rcfile = arg[9:]
-                del args[i]
-                break
-            if arg == '--rcfile':
-                rcfile = args[i + 1]
-                del args[i:i+2]
-                break
+        self._rcfile = None
+        self._plugins = []
+        preprocess_options(args, {'rcfile': self.cb_set_rcfile,
+                                  'load-plugins': self.cb_add_plugins})
         self.linter = linter = PyLinter((
             ('rcfile',
              {'action' : 'callback', 'callback' : lambda *args: 1,
@@ -683,16 +766,24 @@ them in the configuration. This option causes the program to exit'''}),
               'help' : '''Generate a man page for pylint. This option causes \
 the program to exit'''}),
             
+            ('debug-mode',
+             {'action' : 'callback', 'callback' : self.cb_debug_mode,
+              'help' : '''In debug mode, checkers without error messages are \
+disabled and for others, only the ERROR messages are displayed, and no reports \
+are done by default'''}),
+            
             ('profile',
              {'type' : 'yn', 'metavar' : '<y_or_n>',
               'default': False,
               'help' : 'Profiled execution.'}),
 
-            ), reporter=reporter, pylintrc=rcfile)
+            ), reporter=reporter, pylintrc=self._rcfile)
         linter.quiet = quiet
-        # register checkers
+        # register standard checkers
         from pylint import checkers
         checkers.initialize(linter)
+        # load command line plugins
+        linter.load_plugin_modules(self._plugins)
         # add some help section
         linter.add_help_section('Environment variables', config.ENV_HELP)
         linter.add_help_section('Output', '''
@@ -706,8 +797,16 @@ There are 5 kind of message types :
     * (F) fatal, if an error occured which prevented pylint from doing further \
 processing.     
         ''')
-        # load configuration
-        linter.load_file_configuration()
+        # read configuration
+        linter.read_config_file()
+        # is there some additional plugins in the file configuration, in
+        config_parser = linter._config_parser
+        if config_parser.has_option('master', 'load-plugins'):
+            plugins = get_csv(config_parser.get('master', 'load-plugins'))
+            linter.load_plugin_modules(plugins)
+        # now we can load file config and command line, plugins (which can
+        # provide options) have been registered
+        linter.load_config_file()
         args = linter.load_command_line_configuration(args)
         if not args:
             print linter.help()
@@ -728,11 +827,32 @@ processing.
         else:
             linter.check(args)
         sys.path.pop(0)
+
+    def cb_set_rcfile(self, name, value):
+        """callback for option preprocessing (ie before optik parsing)"""
+        self._rcfile = value
         
+    def cb_add_plugins(self, name, value):
+        """callback for option preprocessing (ie before optik parsing)"""
+        self._plugins.extend(get_csv(value))
+                
     def cb_disable_all_checkers(self, *args, **kwargs):
         """optik callback for disabling all checkers"""
         self.linter.disable_all_checkers()
-            
+
+    def cb_debug_mode(self, *args, **kwargs):
+        """debug mode:
+        * checkers without error messages are disabled 
+        * for others, only the ERROR messages are displayed
+        * disable reports
+        * do not save execution information
+        """
+        self.linter.disable_noerror_checkers()
+        self.linter.set_option('disable-msg-cat', ('W', 'C', 'R', 'F', 'I'))
+        self.linter.set_option('reports', False)
+        self.linter.set_option('persistent', False)
+        
+    
     def cb_generate_config(self, *args, **kwargs):
         """optik callback for sample config file generation"""
         self.linter.generate_config()
