@@ -15,7 +15,7 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """functional/non regression tests for pylint"""
 
-__revision__ = '$Id: func_test.py,v 1.36 2005/11/02 09:21:58 syt Exp $'
+__revision__ = '$Id: func_test.py,v 1.37 2005-12-28 14:58:22 syt Exp $'
 
 import unittest
 import sys
@@ -109,9 +109,9 @@ class TestTests(unittest.TestCase):
             todo.remove(msg_id)
         todo.sort()
         if PY23:
-            self.assertEqual(todo, ['E0503', 'F0002', 'F0201', 'F0202', 'F0203', 'F0204', 'F0321', 'I0001'])
+            self.assertEqual(todo, ['E0503', 'F0002', 'F0202', 'F0321', 'I0001'])
         else:
-            self.assertEqual(todo, ['F0002', 'F0201', 'F0202', 'F0203', 'F0204', 'F0321', 'I0001'])
+            self.assertEqual(todo, ['F0002', 'F0202', 'F0321', 'I0001'])
 
         
 def make_tests(filter_rgx):
