@@ -1,6 +1,6 @@
+# pylint:disable-msg=C0103,W0104
 """Check format
 """
-
 __revision__ = ''
 
 notpreceded= 1
@@ -56,3 +56,6 @@ yo+=4
 html2 = """<option value='='>ist genau gleich</option>
 yo+=4
 """
+
+func('''<body>Hello
+</body>''')
