@@ -1,4 +1,4 @@
-# pylint:disable-msg=W0212
+# pylint:disable-msg=R0201
 """Checks that class variables are seen as inherited !
 """
 __revision__ = ''
@@ -31,6 +31,6 @@ class MyClass(BaseClass):
         print self.base_var, self.var
 
 if __name__ == '__main__':
-    obj = MyClass()
-    obj.met()
+    OBJ = MyClass()
+    OBJ.met()
 
