@@ -14,34 +14,34 @@
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
-__revision__ = "$Id: smoketest.py,v 1.5 2005/01/20 15:12:56 syt Exp $"
+__revision__ = "$Id: smoketest.py,v 1.6 2005/04/15 10:40:24 syt Exp $"
 
 import unittest
 import sys
 from cStringIO import StringIO
 
-from logilab.pylint.lint import Run
-from logilab.pylint.reporters.text import TextReporter, TextReporter2, ColorizedTextReporter
-from logilab.pylint.reporters.html import HTMLReporter
+from pylint.lint import Run
+from pylint.reporters.text import TextReporter, TextReporter2, ColorizedTextReporter
+from pylint.reporters.html import HTMLReporter
 
     
 class LintSmokeTest(unittest.TestCase):
         
     def test1(self):
         """make pylint checking itself"""
-        Run(['--include-ids=y', 'logilab.pylint'], reporter=TextReporter(StringIO()), quiet=1)
+        Run(['--include-ids=y', 'pylint'], reporter=TextReporter(StringIO()), quiet=1)
     
     def test2(self):
         """make pylint checking itself"""
-        Run(['logilab.pylint.lint'], reporter=TextReporter2(StringIO()), quiet=1)
+        Run(['pylint.lint'], reporter=TextReporter2(StringIO()), quiet=1)
     
     def test3(self):
         """make pylint checking itself"""
-        Run(['logilab.pylint.checkers'], reporter=HTMLReporter(StringIO()), quiet=1)
+        Run(['pylint.checkers'], reporter=HTMLReporter(StringIO()), quiet=1)
     
     def test4(self):
         """make pylint checking itself"""
-        Run(['logilab.pylint.checkers'], reporter=ColorizedTextReporter(StringIO()), quiet=1)
+        Run(['pylint.checkers'], reporter=ColorizedTextReporter(StringIO()), quiet=1)
     
     def test_generate_config_option(self):
         """make pylint checking itself"""
