@@ -4,8 +4,8 @@ import unittest
 from os.path import exists
 from cStringIO import StringIO
 
-from logilab.pylint.checkers import initialize, imports
-from logilab.pylint.lint import PyLinter
+from pylint.checkers import initialize, imports
+from pylint.lint import PyLinter
 
 from utils import TestReporter
 
