@@ -14,7 +14,7 @@
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
-__revision__ = '$Id: unittest_lint.py,v 1.14 2005/05/27 10:32:26 syt Exp $'
+__revision__ = '$Id: unittest_lint.py,v 1.16 2006-04-19 09:17:40 syt Exp $'
 
 import unittest
 import sys
@@ -108,16 +108,16 @@ class PyLinterTC(unittest.TestCase):
         self.assert_(linter.is_message_enabled('W0101'))
         self.assert_(linter.is_message_enabled('W0102'))
         linter.disable_message('W0101', scope='package')
-        linter.disable_message('W0102', scope='module')
+        linter.disable_message('W0102', scope='module', line=1)
         self.assert_(not linter.is_message_enabled('W0101'))
-        self.assert_(not linter.is_message_enabled('W0102'))
+        self.assert_(not linter.is_message_enabled('W0102', 1))
         linter.set_current_module('tutu')
         self.assert_(not linter.is_message_enabled('W0101'))
         self.assert_(linter.is_message_enabled('W0102'))        
         linter.enable_message('W0101', scope='package')
-        linter.enable_message('W0102', scope='module')
+        linter.enable_message('W0102', scope='module', line=1)
         self.assert_(linter.is_message_enabled('W0101'))
-        self.assert_(linter.is_message_enabled('W0102'))
+        self.assert_(linter.is_message_enabled('W0102', 1))
 
     def test_enable_message_category(self):
         linter = self.linter
@@ -137,7 +137,51 @@ class PyLinterTC(unittest.TestCase):
         self.assert_(linter.is_message_enabled('W0101'))
         self.assert_(linter.is_message_enabled('R0102'))
 
-
+    def test_enable_message_block(self):
+        linter = self.linter
+        linter.open()
+        filepath = join('input', 'func_block_disable_msg.py')
+        linter.set_current_module('func_block_disable_msg')
+        linter.process_module(open(filepath))
+        orig_state = linter._module_msgs_state.copy()
+        linter._module_msgs_state = {}
+        linter.collect_block_lines(linter.get_astng(filepath, 'func_block_disable_msg'), orig_state)
+        # global (module level)
+        self.assert_(linter.is_message_enabled('W0613'))
+        self.assert_(linter.is_message_enabled('E1101'))
+        # meth1
+        self.assert_(linter.is_message_enabled('W0613', 13))
+        # meth2
+        self.assert_(not linter.is_message_enabled('W0613', 18))
+        # meth3
+        self.assert_(not linter.is_message_enabled('E1101', 24))
+        self.assert_(linter.is_message_enabled('E1101', 26))
+        # meth4        
+        self.assert_(not linter.is_message_enabled('E1101', 32))
+        self.assert_(linter.is_message_enabled('E1101', 36))
+        # meth5
+        self.assert_(not linter.is_message_enabled('E1101', 42))
+        self.assert_(not linter.is_message_enabled('E1101', 43))
+        self.assert_(linter.is_message_enabled('E1101', 46))
+        self.assert_(not linter.is_message_enabled('E1101', 49))
+        self.assert_(not linter.is_message_enabled('E1101', 51))
+        # meth6
+        self.assert_(not linter.is_message_enabled('E1101', 57))
+        self.assert_(linter.is_message_enabled('E1101', 61))
+        self.assert_(not linter.is_message_enabled('E1101', 64))
+        self.assert_(not linter.is_message_enabled('E1101', 66))
+        
+        self.assert_(linter.is_message_enabled('E0602', 57))
+        self.assert_(linter.is_message_enabled('E0602', 61))
+        self.assert_(not linter.is_message_enabled('E0602', 62))
+        self.assert_(linter.is_message_enabled('E0602', 64))
+        self.assert_(linter.is_message_enabled('E0602', 66))
+        # meth7
+        self.assert_(not linter.is_message_enabled('E1101', 70))
+        self.assert_(linter.is_message_enabled('E1101', 72))
+        self.assert_(linter.is_message_enabled('E1101', 75))
+        self.assert_(linter.is_message_enabled('E1101', 77))
+        
     def test_list_messages(self):
         sys.stdout = StringIO()
         try:
@@ -168,18 +212,19 @@ class PyLinterTC(unittest.TestCase):
 
     def test_set_option_1(self):
         linter = self.linter
-        linter.set_option('disable-msg', 'W0131,W0142')
-        self.assert_(not linter.is_message_enabled('W0131'))
+        linter.set_option('disable-msg', 'C0111,W0142')
+        self.assert_(not linter.is_message_enabled('C0111'))
         self.assert_(not linter.is_message_enabled('W0142'))
         self.assert_(linter.is_message_enabled('W0113'))
 
     def test_set_option_2(self):
         linter = self.linter
-        linter.set_option('disable-msg', ('W0131', 'W0142') )
-        self.assert_(not linter.is_message_enabled('W0131'))
+        linter.set_option('disable-msg', ('C0111', 'W0142') )
+        self.assert_(not linter.is_message_enabled('C0111'))
         self.assert_(not linter.is_message_enabled('W0142'))
         self.assert_(linter.is_message_enabled('W0113'))
 
+
 from pylint import config
 
 class ConfigTC(unittest.TestCase):
